import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The class Dodo.
 * 
 * @author Sjaak Smetsers
 * @version 02 03 2015
 */
public class Dodo extends MovableActor
{
    private int myDirection;

    private GreenfootImage imageRight, imageLeft;
    
    protected Dodo ( int init_direction ) {
        this.myDirection = modulo( init_direction, 4);
        imageRight  = getImage();
        imageLeft = new GreenfootImage( imageRight );
        imageLeft.mirrorHorizontally();
        setImage ();

    }
    
    private void setImage (){
        if ( myDirection == NORTH ) {
            setImage( imageLeft );
            setRotation( 90 );
        } else if ( myDirection == EAST ) {
            setImage( imageRight );
            setRotation( 0 );
        } else if ( myDirection == SOUTH ) {
            setImage( imageRight );
            setRotation( 90 );
        } else if ( myDirection == WEST ) {
            setImage( imageLeft );
            setRotation( 0 );
        } 
    }
    
    
    public void updateScores( int score1, int score2 ) {
        Referee.getReferee().updateScoreboard( score1, score2 );
    }
            
    public boolean fenceAhead() {
        return getActorAhead( Fence.class ) != null;
    }

    public boolean eggAhead() {
        return getEggAhead(  ) != null;
    }
    
    public boolean nestAhead() {
        return getActorAhead( Nest.class ) != null;
    }

    public Nest getNestAhead() {
        return getActorAhead( Nest.class );
    }

    public boolean foundEgg() {
        return getEgg( ) != null;
    }

    public void layEgg() {
        getWorld().addObject( new BlueEgg (), getX (), getY () );
    }

    public Egg getEgg() {
        return getActor( Egg.class );
    }

    public Egg getEggAhead() {
        return getActorAhead( Egg.class );
    }


    public void setDirection( int new_direction ){
        new_direction = modulo( new_direction, 4 );
        if ( this.myDirection != new_direction ) {
            this.myDirection = new_direction;
            setImage();
            if ( Madagaskar.traceIsOn() ) {
                Greenfoot.delay(1);
            }
        }
    }

    public int randomDirection() {
        return Greenfoot.getRandomNumber( 4 );
    }
    
    private int modulo( int a, int b ) {
        return (a % b + b) % b;
    }

    public int getDirection(){
        return myDirection;
    }

    private <E extends Actor> E getActorAhead(Class<E> cls){
        return getActorAhead( myDirection, cls );
    }


    public void step() {
        step( myDirection );
        if ( Madagaskar.traceIsOn() ) {
            Greenfoot.delay(1);
        }
    }
    
    /**
     * Test is we are facing the border.
     */
    public boolean borderAhead() {
        return borderAhead( myDirection );
    }

    public boolean facingNorth () {
        return getDirection() == NORTH;
    }
    
    /**
     * Turns towards the left.
     */
    public void turnLeft() {
        setDirection( modulo( myDirection-1, 4 ) );
    }

    /**
     * Turns towards the right.
     */
    public void turnRight() {
        setDirection( modulo( myDirection+1, 4 ) );
    }

    protected void showCompliment ( String compl_msg ) {
        if ( ! Message.messageActive( getWorld() ) ) {
            World my_world = getWorld();
            my_world.addObject (new Compliment ( compl_msg ),my_world.getWidth()/2, my_world.getHeight()/2);
        }
    }


}
